#ifndef ReadSource_H
#define ReadSource_H


/* A set of procedures for reading text files.

   Copyright 1988,1989,1991  Capilano Computing Systems Ltd.
   All rights reserved.

   By Chris Dewhurst
   
   	10/25/95 CD	Added routine CompareSrc
*/
#include <Carbon/Carbon.h>

#include "CDSys.h"	/* Defines which system/compiler we're running on */

#define DIRECTORY short

#ifdef __cplusplus
extern "C" {
#endif


typedef enum
{
	srcStmt, srcLine
} tSrcMode;

typedef struct tCharSet
{
	short bs[8];
	Boolean empty;
} tCharSet;


/* "gCurSrcChar" at all times contains the character in the current
   position in the file.  If the end of a statement or the file
   have been reached, gCurSrcChar will be set to 0C. */

extern char gCurSrcChar;

extern Boolean gEndOfSrcStmt;	/* true IF past END of line */
extern Boolean gEndOfSrcFile;	/* true IF past END of file */
extern tCharSet gNameChars;	/* A set initialized to alphanumeric chars */
extern tCharSet gAllChars;	/* A set initialized to empty */
extern tCharSet gNumChars;	/* A set initialized to "0123456789ABCDEF-" */



/* Note re comments:

        ReadSource ignores everything between a "{" and the corresponding
        "}", which can be nested, unless "NextEscChar" is called.
*/



/* "InitCharSet" sets up a character set for use in GetSrcToken.
   The characters in the string are placed in the set, which allows
   more efficient comparisons */

pQUALIFIER void InitCharSet(tCharSet *charSet, char chars[], short maxLen);



/* "NextSrcChar" reads the next character from the source file into
   the global variable "gCurSrcChar" unless the end of a statement
   (gEndOfSrcStmt set) or the end of the file (gEndOfSrcFile set) has been
   reached.  If the end of statement or file have been reached,
   gCurSrcChar is set to 0C. */

pQUALIFIER void NextSrcChar(void);


/* "NextEscChar" reads the next character from the source file into
   the global variable "gCurSrcChar" unless the end of the file
   (gEndOfSrcFile set) has been reached.  If the end of file has been reached,
   gCurSrcChar is set to 0C.  This call does not check for comments
   ("{" or "}"), carriage returns, end of statement markers, or
   nulls, as is done by "NextSrcChar".  This is primarily intended
   to implement an "escape character" mechanism. */

pQUALIFIER void NextEscChar(void);



/* "CurSrcChar" returns the current character (see comments above) */

pQUALIFIER char CurSrcChar(void);


/* "EndOfSrcStmt" returns TRUE if the current character position
   is past the end of the statement */

pQUALIFIER Boolean EndOfSrcStmt(void);



/* "EndOfSrcFile" returns TRUE if the current character position
   is past the end of the file */

pQUALIFIER Boolean EndOfSrcFile(void);



/* "NextSrcStmt" moves the source file position to the first character
   in the next statement. */

pQUALIFIER void NextSrcStmt(void);


/* "GetSrcPosn" returns the current character position within the source
   file.  This number can be passed to SrcSeek to reposition the file. */

pQUALIFIER long GetSrcPosn(void);



/* "SrcSeek" repositions the file to the given point. */

pQUALIFIER void SrcSeek(long newPos);


/* "BackspaceSrc" backs up the source file pointer to the position it
 was at before the last call that moved it */

pQUALIFIER void BackspaceSrc(void);


/* Moves through the text file until "gCurSrcChar" is a non-blank or
   "gEndOfSrcFile" or "gEndOfSrcStmt" is reached.  If "gCurSrcChar" is already
   non-blank, nothing is done. */

pQUALIFIER void SkipSrcBlanks(void);



/* "CompareSrc" compares the given constant, null-terminated string with the contents
   of the source file at the current position.
   
   If the string matches, TRUE is returned and the file is positioned at
   the next character past the matched string.
   
   If the string doesn't match, FALSES is returned and the file position
   is not changed.
   
   NOTE: "CompareSrc" only works on null-terminated strings!
*/
pQUALIFIER Boolean CompareSrc(char	*src);



/* "GetSrcToken" skips blanks in the file starting at the current
   position, then proceeds copying characters from the file into
   "item" until one is found that is not in "charSet" or the length
   of the argument "item" is exceeded.  If "charSet"
   is empty, then all non-blank characters are read.
   "GetSrcToken" returns TRUE if any characters are copied into "Item" */

pQUALIFIER Boolean GetSrcToken(char *item_, tCharSet charSet, short maxLen);



/* "GetSrcValue" reads a decimal INTEGER value from -32767 to 32767
   from the file and returns TRUE if any is found.  The number is
   terminated by any non-numeric character. */

pQUALIFIER Boolean GetSrcValue(short *value);


/* "GetSrcLong" reads a LONGINT value in decimal, hex or binary
   from the file and returns TRUE if any is found.  The number is
   terminated by any non-numeric character.  Hex numbers are written
   nnnH and binary nnnB */

pQUALIFIER Boolean GetSrcLong(long *value);


/* "GetSrcLineNum" returns the current line number in the current source file.
   This is intended for error reporting purposes and includes line breaks in comments.
   The line number is invalidated by a "SrcSeek" operation and will return 0 after
   a seek to anything other than zero. */

pQUALIFIER short GetSrcLineNum(void);



/* "GetSrcBlock" returns a pointer to the internal buffer containing
   the desired range of characters in the current source.
   "startPos" and "endPos" should be values returned by GetSrcPosn.
   If they are out of range, they will be adjusted to the beginning
   or end of the file, as appropriate.  If endPos < startPos, the block
   length will be returned as zero.

   "pBlock" will return a pointer to the first character,
   "blockLen" will return its length.  Note that the block is the verbatim
   contents of the source file, including any comments, escape sequences, etc. */

pQUALIFIER void GetSrcBlock(long	startPos,
							long	endPos,
							char	**pBlock,
							long	*blockLen);



/* "SrcInit" opens a text file and must be called before any other
   routines in this module.  If "blockAddr" is non-NIL then it is assumed
   to point to a block of text which is used instead of a file.
   If fileName is an empty string, it puts
   up a file request to allow the user to select a file.  "mode" must
   be one of "srcStmt" or "srcLine".  If it is "srcStmt", line breaks
   are ignored (returned as a blank) and statements are terminated by
   a ";".  If it is "srcLine", statements are terminated by a carriage
   return.  If "echo" is TRUE, all source is echoed to terminal.  If
   "passControlChars" is TRUE then control characters will be returned
   in "gCurSrcChar", otherwise they are converted to blanks. */

pQUALIFIER short SrcInit(FSRefPtr  setUpFSSpec,
                         char       *promptString,
						 Boolean	prompt,
						 void		*blockAddr,
						 long		blockLen,
						 tSrcMode	mode,
						 Boolean	echo,
						 Boolean	passControlChars);


// 9/20/96 CD SrcInitNest is like SrcInit except that it does not
// close any existing open record, but adds itself to a list of
// open files and makes the new one current.

pQUALIFIER short SrcInitNest(FSRefPtr  setUpFSSpec,
                             char       *promptString,
							 Boolean	prompt,
							 void		*blockAddr,
							 long		blockLen,
							 tSrcMode	mode,
							 Boolean	echo,
							 Boolean	passControlChars);



/* "SrcClose" closes the source file and deallocates any memory used */

pQUALIFIER void SrcClose(void);


/* 9/20/96 CD SrcCloseNest closes only the currently open source file
   and sets the previous open file (if any) to be current. */

pQUALIFIER void SrcCloseNest(void);



/* InitCDRdSrc must be called before calling any others. */

pQUALIFIER void InitCDRdSrc(void);


pQUALIFIER void CDRdSrc_Init(void);


#ifdef __cplusplus
}
#endif


#endif /*ReadSource_H*/


